<?php
/*
Plugin Name: Category Templates
Plugin URI: http://aarongloege.com/blog/web-development/wordpress/plugins/wordpress-plugin-category-templates/
Description: Gives you the ability to apply different templates to your posts either individually, or by category.
Version: 1.0.2
Author: Aaron Gloege
Author URI: http://www.aarongloege.com/

===============================================================================

Copyright 2009  Aaron Gloege  (contact@aarongloege.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

===============================================================================
*/

function cat_temp_menus() {
	add_submenu_page('themes.php','Category Templates', 'Category Templates', 8, basename(__FILE__), 'cat_temp_options_page');
	if (function_exists('add_meta_box')) {
		add_meta_box('cat_template_select','Post Template','cat_temp_meta','post','side','low');
	}
}

function cat_temp_meta() {
	global $wpdb, $post_ID;
	
	$templates = cat_temp_get_page_templates();
	$current = cat_temp_post_template($post_ID);

	$out = '<select name="cat_temp_template" style="width:100%">';
	$out .= '<option value="none"';
	if ($post_ID == 0 || !$current) $out .= ' selected="selected"';
	$out .= '>No Template</option>';
	$out .= '<option value="/single.php"';
	if ($current == "/single.php") $out .= ' selected="selected"';
	$out .= '>Default Template</option>';
	foreach ($templates as $template => $file) {
		$out .= '<option value="'.$file.'"';
	if ($current == $file) $out .= ' selected="selected"';
	$out .= '>'.$template.'</option>';
	}
	$out .= "</select>";
	$out .= "<p>Choosing a template here will override any templates assigned to this post's categories.</p>";
	echo $out;
}

function cat_temp_post_submit($post_ID) {
  global $wpdb;
  if ($_POST['cat_temp_template']) {
		$templates = unserialize(get_option("cat_temp_post"));
		if ($_POST['cat_temp_template'] != 'none') {
			$templates[$post_ID] = $_POST['cat_temp_template'];
		} else {
			if ($templates[$post_ID]) {
				unset($templates[$post_ID]);
			}
		}
		update_option("cat_temp_post",serialize($templates));
  }
}

function cat_temp_post_template($ID) {
	$templates = unserialize(get_option("cat_temp_post"));
	return $templates[$ID];
}

function cat_temp($template) {
	global $wp_query;
	$post_obj = $wp_query->get_queried_object();
	$categories = get_the_category($post_obj->ID);
	
	$data = cat_temp_get_data();
	$temp_data;
	
	// Get templates for post categories
	foreach($categories as $category) {
		foreach($data as $cat) {
			if($category->term_id == $cat['id'] && $cat['template'] != "0") {
				$temp_data[$cat['template']] = $cat['priority']+($category->term_id/80000);
			}
		}
	}
	
	// Get templates for parent categories
	foreach($data as $cat) {
		$go = cat_temp_is_cat($cat['id']);
		if ($cat['all'] == "all" && $cat['template'] != "0") {
			$descendants = get_term_children((int)$cat['id'], 'category');
			if ($descendants && in_category($descendants)) {
				$temp_data[$cat['template']] = $cat['priority']+($cat['id']/80000);
			}
		}
	}
	
	//print_r($temp_data);
	
	// Sort templates by priotiry, and return the one with the highest priority
	if(is_array($temp_data)) {
		asort($temp_data);
		$template = array_shift(array_keys($temp_data));
	}
	
	// If currrent post has a template, use that one instead
	$overRule = cat_temp_post_template($post_obj->ID);
	if($overRule) $template = $overRule;
	
	// Return template path
	if(!empty($template)) {
		if (file_exists(TEMPLATEPATH."$template")) {
			return TEMPLATEPATH."$template";
		}
	}
}

function cat_temp_is_cat($cat, $_post = null) {
	if (in_category($cat, $_post)) {
		return true;
	} else {
		$descendants = get_term_children((int)$cat, 'category');
		if ($descendants && in_category($descendants, $_post)) return true;
	}
	return false;
}


function cat_temp_get_page_templates() {
	$themes = get_themes();
	$theme = get_current_theme();
	$templates = $themes[$theme]['Template Files'];
	$page_templates = array();
	
	if (is_array($templates)) {
		foreach($templates as $template) {
			$template_data = implode('', file( WP_CONTENT_DIR.$template));
		
			$name = '';
			if (preg_match('|Template Name:(.*)$|mi', $template_data, $name))
				$name = $name[1];
		
			if (!empty($name)) {
				$page_templates[trim($name)] = str_ireplace($themes[$theme]['Template Dir'], "", $template);// basename( $template );
			}
		}
	}
	return $page_templates;
}

function cat_temp_cats($item, $current) {
	$templates = cat_temp_get_page_templates();
	$out = '<select name="data['.$item.'][template]">';
	$out .= '<option value="0"';
	if ($current == "0") $out .= ' selected="selected"';
	$out .= '>No Template</option>';
	$out .= '<option value="/single.php"';
	if ($current == "/single.php") $out .= ' selected="selected"';
	$out .= '>Default Template</option>';
	foreach ($templates as $template => $file) {
		$out .= '<option value="'.$file.'"';
	if ($current == $file) $out .= ' selected="selected"';
	$out .= '>'.$template.'</option>';
	}
	$out .= "</select>";
	return $out;
}

function cat_temp_categories($child = 0) {
	$data = array(
		"hide_empty" => false,
		"child_of" => $child,
		"pad_count" => false,
	);
	$categories = get_categories($data);
	
	$list = array();
	foreach ($categories as $cat) {
		if ($cat->parent == $child) {
			$list[] = array(
				"name" => $cat->name,
				"id" => $cat->cat_ID,
				"count" => cat_temp_getALLposts($cat->cat_ID),
				"acount" => $cat->category_count,
				"child" => cat_temp_categories($cat->cat_ID),
			);
		}
	}
	return $list;
}

function cat_temp_li_fun($data) {
	$out = "<ul class=\"cat-temp\">";
	foreach ($data as $root) {
		$out .= "<li>".$root['name']." (".$root['count'].")</li>";
		if (count($root['child']) > 0) {
			$out .= cat_temp_li_fun($root['child']);
		}
		//$out .= "</li>";
	}
	$out .= "</ul>";
	return $out;
}

function cat_temp_priority($item, $current) {
	$pri = array("Lowest","Low","Medium","High","Highest");
	$out = '<select name="data['.$item.'][priority]">';
	$t = 0;
	for ($i = 10; $i >= 1; $i = $i-2) {
		$out .= '<option value="'.$i.'"';
		if (intval($current) == $i) $out .= ' selected="selected"';
		$out .= '>'.$pri[$t].'</option>';
		$t++;
	}
	$out .= "</select>";
	return $out;
}

function cat_temp_getALLposts($ID) {
	$td = array(
		'numberposts' => -1,
		'category' => $ID,
	);
	return count(get_posts($td));

}

function cat_temp_admin_head() {
	echo '<link type="text/css" rel="stylesheet" media="all" href="'.plugins_url('category-templates/admin.css').'" />';
}

function cat_temp_get_data($id=false) {
	if (!$id) {
		return unserialize(get_option('cat_temp_data'));
	} else {
		$t = unserialize(get_option('cat_temp_data'));
		return $t[$id];
	}
}

function cat_temp_update($data) {
	return update_option('cat_temp_data', serialize($data));
}

function cat_temp_delete() {
	delete_option('cat_temp_data');
	delete_option('cat_temp_post');
}

function cat_temp_td_fun($data, $padding = 5) {
	$out = "";
	foreach ($data as $root) {
		$tr = cat_temp_get_data($root['id']);
		$out .= "<tr>";
		$out .= "<td class='c'>".$root['id']."<input type=\"hidden\" name=\"data[".$root['id']."][id]\" value=\"".$root['id']."\" /></td>";
		$out .= "<td class=\"wide\" style=\"padding-left:".$padding."px;\">".__($root['name'])."</td>";
		$out .= "<td>".cat_temp_cats($root['id'], $tr['template']).'&nbsp;<input type="checkbox"';
		if ($tr['all'] == "all") $out .= ' checked="checked"';
		$out .= ' name="data['.$root['id'].'][all]" value="all" />&nbsp<small>'.__('Apply to sub-categories').'</small></td>';
		$out .= "<td class=\"c\">".cat_temp_priority($root['id'], $tr['priority'])."</td>";
		$out .= "<td>".$root['acount']."<span style=\"color:#a4a4a4;\">(".$root['count'].")</span></td>";
		$out .= "</tr>";
		if (count($root['child']) > 0) {
			$out .= cat_temp_td_fun($root['child'], $padding+20);
		}
	}
	return $out;
}

function cat_temp_options_page() {
?>

<div class="wrap cat-template">
	<div class="icon32" id="icon-themes"><br/></div>
	<h2><?php _e('Settings for Category Templates');?></h2>
	<p><?php _e('Configure the options below to apply templates to your categories.');?></p>
	<?php
if ($_POST['update_theme']) {
	cat_temp_update($_POST['data']);
	echo '<div id="message" class="updated"><p>Theme Settings Updated</p></div>';
}
	?>
	<form method="post" action="<?php echo $_SERVER['PHP_SELF']."?".$_SERVER['QUERY_STRING'];?>">
    <?php
        // New way of setting the fields, for WP 2.7 and newer
        if(function_exists('settings_fields')){
            settings_fields('cat-temp-options');
        } else {
            wp_nonce_field('update-options');
            ?>
            <input type="hidden" name="action" value="update" />
            <?php
        }
    ?>
    <table width="100%" class="cat-holder">
    	<tr>
    		<td width="70%"><table class="cat-temp" cellspacing="0" border="0">
					<thead>
						<tr>
							<th width="40" class="c">ID</th>
							<th><?php _e('Category');?></th>
							<th><?php _e('Template');?></th>
							<th class="c"><?php _e('Priority');?></th>
							<th><?php _e("Post's");?></th>
						</tr>
					</thead>
					<tbody>
						<?php echo cat_temp_td_fun(cat_temp_categories());?>
					</tbody>
				</table></td>
				<td width="30%"><table class="cat-temp direc" cellspacing="0" border="0">
					<thead>
						<tr>
							<th><?php _e('Directions');?></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td><p><?php _e('To specify a template for a certain category, select the template from the appropriate drop down box. The templates listed are the standard Wordpress page templates.');?></p>
							<p><?php _e('If not already done, you must add the following code to the top of your templates for a template to appear here.');?></p>
							<code>&lt;?php /* Template Name: Blog Template */ ?&gt;</code>
							<p><?php _e("Checking the <strong>Apply to sub-categories</strong> box does just that: the template is applied to all posts within its category <em>and</em> sub-categories. Templates will only apply to sub-categories that have no templates assigned, or if its of higher priority.");?></p>
							<p><?php _e("Choosing a priority will tell Wordpress which template to favor if the post is in two categories that has templates assigned to each of them. If priorities are the same, the category with the lowest ID will have its template applied.");?></p></td>
						</tr>
					</tbody>
				</table></td>
			</tr>
		</table>
    <p class="submit"><input type="submit" name="update_theme" value="<?php _e('Save Changes') ?>" /></p>
	</form>
</div>
<?php

}

add_action('admin_head', 'cat_temp_admin_head');
add_action('admin_menu', 'cat_temp_menus');
add_action('save_post', 'cat_temp_post_submit');
add_filter('single_template', 'cat_temp');

?>